"""Module that updates the domain model of :term:`GSO`. Should contain all types of subscriptions.

.. warning::
   Whenever a new product is added, this should be reflected in the :py:class:`gso.products.ProductType` enumerator.
   This does not hold for adding a new type of already existing product.
"""

from orchestrator.domain import SUBSCRIPTION_MODEL_REGISTRY
from pydantic_forms.types import strEnum

from gso.products.product_types.edge_port import EdgePort
from gso.products.product_types.iptrunk import ImportedIptrunk, Iptrunk
from gso.products.product_types.lan_switch_interconnect import LanSwitchInterconnect
from gso.products.product_types.office_router import ImportedOfficeRouter, OfficeRouter
from gso.products.product_types.opengear import ImportedOpengear, Opengear
from gso.products.product_types.pop_vlan import PopVlan
from gso.products.product_types.router import ImportedRouter, Router
from gso.products.product_types.site import ImportedSite, Site
from gso.products.product_types.super_pop_switch import ImportedSuperPopSwitch, SuperPopSwitch
from gso.products.product_types.switch import Switch


class ProductName(strEnum):
    """An enumerator of available product names in :term:`GSO`."""

    IP_TRUNK = "IP trunk"
    ROUTER = "Router"
    SITE = "Site"
    SUPER_POP_SWITCH = "Super PoP switch"
    OFFICE_ROUTER = "Office router"
    SWITCH = "Switch"
    LAN_SWITCH_INTERCONNECT = "LAN Switch Interconnect"
    POP_VLAN = "Pop VLAN"
    IMPORTED_IP_TRUNK = "Imported IP trunk"
    IMPORTED_ROUTER = "Imported router"
    IMPORTED_SITE = "Imported site"
    IMPORTED_SUPER_POP_SWITCH = "Imported super PoP switch"
    IMPORTED_OFFICE_ROUTER = "Imported office router"
    OPENGEAR = "Opengear"
    IMPORTED_OPENGEAR = "Imported Opengear"
    EDGE_PORT = "Edge port"


class ProductType(strEnum):
    """An enumerator of available product types in :term:`GSO`."""

    IP_TRUNK = Iptrunk.__name__
    ROUTER = Router.__name__
    SITE = Site.__name__
    SUPER_POP_SWITCH = SuperPopSwitch.__name__
    OFFICE_ROUTER = OfficeRouter.__name__
    SWITCH = Switch.__name__
    LAN_SWITCH_INTERCONNECT = LanSwitchInterconnect.__name__
    POP_VLAN = PopVlan.__name__
    IMPORTED_IP_TRUNK = ImportedIptrunk.__name__
    IMPORTED_ROUTER = ImportedRouter.__name__
    IMPORTED_SITE = ImportedSite.__name__
    IMPORTED_SUPER_POP_SWITCH = ImportedSuperPopSwitch.__name__
    IMPORTED_OFFICE_ROUTER = ImportedOfficeRouter.__name__
    OPENGEAR = Opengear.__name__
    IMPORTED_OPENGEAR = Opengear.__name__
    EDGE_PORT = EdgePort.__name__


SUBSCRIPTION_MODEL_REGISTRY.update(
    {
        ProductName.IP_TRUNK.value: Iptrunk,
        ProductName.ROUTER.value: Router,
        ProductName.SITE.value: Site,
        ProductName.SUPER_POP_SWITCH.value: SuperPopSwitch,
        ProductName.OFFICE_ROUTER.value: OfficeRouter,
        ProductName.SWITCH.value: Switch,
        ProductName.LAN_SWITCH_INTERCONNECT.value: LanSwitchInterconnect,
        ProductName.POP_VLAN.value: PopVlan,
        ProductName.IMPORTED_IP_TRUNK.value: ImportedIptrunk,
        ProductName.IMPORTED_ROUTER.value: ImportedRouter,
        ProductName.IMPORTED_SITE.value: ImportedSite,
        ProductName.IMPORTED_SUPER_POP_SWITCH.value: ImportedSuperPopSwitch,
        ProductName.IMPORTED_OFFICE_ROUTER.value: ImportedOfficeRouter,
        ProductName.OPENGEAR.value: Opengear,
        ProductName.IMPORTED_OPENGEAR.value: ImportedOpengear,
        ProductName.EDGE_PORT.value: EdgePort,
    },
)
